/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.entity.adapter;

import java.util.List;

import mir.entity.Entity;
import mir.entity.EntityBrowser;
import mir.storage.DatabaseFailure;
import mir.util.RewindableIterator;

public class EntityIteratorAdapter implements RewindableIterator {
  private String definitionName;
  private RewindableIterator iterator;
  private EntityAdapterModel model;

  public EntityIteratorAdapter(RewindableIterator anIterator, EntityAdapterModel aModel, String aDefinitionName) {
    model = aModel;
    iterator = anIterator;
    definitionName = aDefinitionName;
  }

  public EntityIteratorAdapter(String aWhereClause, String anOrderByClause,
          int aBatchSize, EntityAdapterModel aModel, String aDefinitionName )
          throws DatabaseFailure {
    this(new EntityBrowser(aModel.getMappingForName(aDefinitionName).getDatabase(), aWhereClause, anOrderByClause, aBatchSize), aModel, aDefinitionName);
  }

  public EntityIteratorAdapter(String aWhereClause, String anOrderByClause,
          int aBatchSize, EntityAdapterModel aModel, String aDefinitionName,
          int aLimit, int aSkip) throws DatabaseFailure {
    this(new EntityBrowser(aModel.getMappingForName(aDefinitionName).getDatabase(), aWhereClause, anOrderByClause, aBatchSize, aLimit, aSkip), aModel, aDefinitionName);
  }

  public EntityIteratorAdapter(String aMainTablePrefix, List someExtraTables,
          String aWhereClause, String anOrderByClause,
          int aBatchSize, EntityAdapterModel aModel, String aDefinitionName,
          int aLimit, int aSkip) throws DatabaseFailure {
    this(new EntityBrowser(aModel.getMappingForName(aDefinitionName).getDatabase(), aMainTablePrefix, someExtraTables, aWhereClause, anOrderByClause, aBatchSize, aLimit, aSkip), aModel, aDefinitionName);
  }


  public boolean hasNext() {
    return iterator.hasNext();
  }

  public Object next() {
    return model.makeEntityAdapter(definitionName, (Entity) iterator.next());
  }

  public void remove() {
    iterator.remove();
  }

  public void rewind() {
    iterator.rewind();
  }
}
