/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.servlet;

import mir.generator.Generator;
import mir.producer.ProducerFactory;
import mir.servlet.AdminServletModule;
import mir.servlet.ServletModuleFailure;
import mir.util.HTTPRequestParser;
import mircoders.global.MirGlobal;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class ServletModuleProducer extends AdminServletModule {
  public void defaultAction(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    showProducerQueueStatus(aRequest, aResponse);
  }

  /**
   *
   * @param aRequest
   * @param aResponse
   */
  public void showProducerQueueStatus(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    try {
      // TODO: use ServletHelper
      Generator generator = MirGlobal.localizer().generators().
          makeAdminGeneratorLibrary().makeGenerator("producerqueue.template", null);

      Map generationData = ServletHelper.makeGenerationData(aRequest, aResponse, new Locale[] { getLocale(aRequest), getFallbackLocale(aRequest)});
      generationData.put( "thisurl", "module=Producer&do=showProducerQueueStatus");

      List producersData = new ArrayList();
      Iterator i = MirGlobal.getProducerEngine().getFactories().iterator();
      while (i.hasNext()) {
        ProducerFactory factory = (ProducerFactory) i.next();

        List producerVerbs = new ArrayList();
        Iterator j = factory.verbs().iterator();
        while (j.hasNext()) {
          Map verbData = new HashMap();
          ProducerFactory.ProducerVerb verb = (ProducerFactory.ProducerVerb) j.next();
          verbData.put("name", verb.getName());
          verbData.put("description", verb.getDescription());

          producerVerbs.add(verbData);
        }

        Map producerData = new HashMap();
        producerData.put("name", factory.getName());
        producerData.put("verbs", producerVerbs);

        producersData.add(producerData);
      }
      generationData.put("producers", producersData);

      generationData.put("queue", MirGlobal.getProducerEngine().getQueueStatus());
      generator.generate(aResponse.getWriter(), generationData, getLogger());
    }
    catch (Throwable t) {
      throw new ServletModuleFailure(t);
    }
  }

  /**
   * This method will only be called by external scripts (e.g. from cron jobs).
   * The output therefore is very simple.
   */
  public void produce(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    PrintWriter out;
    try {
      out = aResponse.getWriter();
    }
    catch (IOException e) {
      throw new ServletModuleFailure(e);
    }

    if (aRequest.getParameter("producer")!=null) {
      String producerParam = aRequest.getParameter("producer");
      String verbParam = aRequest.getParameter("verb");

      try {
        MirGlobal.getProducerEngine().addJob(producerParam, verbParam);

        out.println("job added");
      }
      catch (Throwable t) {

        out.println("Can't add job: " + t.getMessage());
      }
    }
  }

  /**
   * Servlet action to propduce a preconfigured combination of producers
   * (e.g. "generate all new")
   */
  public void producerecipe(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    try {
      String recipe = aRequest.getParameter("recipe");
      MirGlobal.localizer().producers().produceRecipe(recipe);

      ServletHelper.generateInfoMessage(aRequest, aResponse,
          new Locale[] { getLocale(aRequest), getFallbackLocale(aRequest)},
           "bundles/admin", "etc/bundles/adminlocal", "recipeAddedToQueue", recipe, "");
    }
    catch (Throwable t) {
      throw new ServletModuleFailure(t);
    }
  }

  /**
   * Servlet action to enqueue a producer job
   */
  public void enqueue(HttpServletRequest aRequest, HttpServletResponse aResponse) {
    try {
      if (aRequest.getParameter("producer")!=null) {
        String producerParam = aRequest.getParameter("producer");
        String verbParam = aRequest.getParameter("verb");

        MirGlobal.getProducerEngine().addJob(producerParam, verbParam);

        ServletHelper.redirect(aResponse, "Producer", "showProducerQueueStatus");
      }
    }
    catch (Throwable t) {
      throw new ServletModuleFailure(t);
    }
  }

  public void cancel(HttpServletRequest aRequest, HttpServletResponse aResponse)  {
    try {
      HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);

      if (requestParser.getParameter("cancelall") != null) {
        MirGlobal.getProducerEngine().cancelAllJobs();
      }
      else {
        List jobs = new ArrayList(requestParser.getParameterList("jobid"));

        MirGlobal.getProducerEngine().cancelJobs(jobs);
      }
      ServletHelper.redirect(aResponse, "Producer", "showProducerQueueStatus");
    }
    catch (Throwable t) {
      throw new ServletModuleFailure(t);
    }
  }
}
