/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.storage;

import java.sql.Connection;
import java.sql.Statement;
import java.util.ArrayList;

import mir.entity.EntityList;
import mir.log.LoggerWrapper;
import mir.storage.Database;
import mir.storage.DatabaseFailure;
import mircoders.entity.EntityUploadedMedia;

public class DatabaseCommentToMedia extends Database {

  private static DatabaseCommentToMedia instance;

  public static synchronized DatabaseCommentToMedia getInstance() {
    if (instance == null) {
      instance = new DatabaseCommentToMedia();
    }
    return instance;
  }

  private DatabaseCommentToMedia() {
    super();

    logger = new LoggerWrapper("Database.CommentToMedia");
    mainTable = "comment_x_media";
    entityClass = mir.entity.GenericEntity.class;
  }

  public void addMedia(String commentId, String mediaId) throws
      DatabaseFailure {
    if (commentId == null && mediaId == null) {
      return;
    }

    Connection con = null;
    Statement stmt = null;
    //now insert

    String sql = "insert into " + mainTable + " (comment_id,media_id) values ("
        + commentId + "," + mediaId + ")";
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- add media failed -- insert");
      throw new DatabaseFailure("-- add media failed -- insert ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void setMedia(String commentId, String mediaId) throws
      DatabaseFailure {
    if (commentId == null && mediaId == null) {
      return;
    }
    //first delete all row with comment_id=commentId
    String sql = "delete from " + mainTable + " where comment_id=" + commentId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- set media failed -- delete");
      throw new DatabaseFailure("-- set media failed -- delete ", e);
    }
    finally {
      freeConnection(con, stmt);
    }

    //now insert
    //first delete all row with comment_id=commentId

    sql = "insert into " + mainTable + " (comment_id,media_id) values ("
        + commentId + "," + mediaId + ")";
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- set media failed -- insert");
      throw new DatabaseFailure("-- set media failed -- insert ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void deleteByCommentId(String commentId) throws DatabaseFailure {
    if (commentId == null) {
      //theLog.printDebugInfo("-- delete topics failed -- no comment id");
      return;
    }
    //delete all row with comment_id=commentId
    String sql = "delete from " + mainTable + " where comment_id=" + commentId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- delete by commentId failed  ");
      throw new DatabaseFailure(
          "-- delete by comment id failed -- delete ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void deleteByMediaId(String mediaId) throws DatabaseFailure {
    if (mediaId == null) {
      //theLog.printDebugInfo("-- delete topics failed -- no topic id");
      return;
    }
    //delete all row with comment_id=commentId
    String sql = "delete from " + mainTable + " where media_id=" + mediaId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
      logger.debug("-- delete media success ");
    }
    catch (Exception e) {
      logger.error("-- delete media failed ");
      throw new DatabaseFailure("-- delete by media id failed -- ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void delete(String commentId, String mediaId) throws
      DatabaseFailure {
    if (mediaId == null || commentId == null) {
      logger.debug("-- delete media failed -- missing parameter");
      return;
    }
    //delete all row with comment_id=commentId and media_id=mediaId
    String sql = "delete from " + mainTable + " where media_id=" + mediaId +
        " and comment_id= " + commentId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
      logger.debug("-- delete comment_x_media success ");
    }
    catch (Exception e) {
      logger.error("-- delete comment_x_media failed ");
      throw new DatabaseFailure("-- delete comment_x_media failed -- ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public EntityList getComment(EntityUploadedMedia media) throws
      DatabaseFailure {

    EntityList returnList = null;
    if (media != null) {

      String id = media.getId();
      ArrayList extraTables = new ArrayList();
      extraTables.add(mainTable + " cxm");

      String mediaSelect = "cxm.comment_id=c.id and cxm.media_id="+id;
      try {
        returnList = DatabaseComment.getInstance().selectByWhereClause("c",
          extraTables, mediaSelect, "c.id" );

      }
      catch (Exception e) {
        logger.error("-- get comment failed");
        throw new DatabaseFailure("-- get comment failed -- ", e);
      }
    }
    return returnList;
  }

  /**
   * Returns a EntityList with all comment-objects having
   *  a relation to a media
   */

  public EntityList getComment() throws DatabaseFailure {
    EntityList returnList = null;

    ArrayList extraTables = new ArrayList();
    extraTables.add(mainTable + " cxm");

    String mediaSelect = "cxm.comment_id=c.id";
    try {
      returnList = DatabaseComment.getInstance().selectByWhereClause("c",
        extraTables, mediaSelect, "c.webdb_lastchange desc" );

    }
    catch (Exception e) {
      logger.error("-- get comment failed");
      throw new DatabaseFailure("-- get comment failed -- ", e);
    }
    return returnList;

  }

}
